/**
 * @brief 708 reader class definition.
 * @file read708.h
 * @version 1.5.1
 * @date March 4, 2016
 * @details Class for reading ARINC 708 data from a file. The base class
 * implements the ARINC 708 standard.
 * 
 * Version History:
 * Version 0.2 on December 23, 2013
 * Version 0.3 on January 6, 2014
 * Version 1.0 on January 13, 2014
 * Version 1.1 on March 6, 2014
 * Version 1.5.1 on March 4, 2016
 */

#ifndef READ708_H
#define	READ708_H

#define HEADERWORDS 4     // Number of header words
#define NRANGBINS  512    // Number of range bins

#include <string>
#include <stdexcept>

using namespace std;

namespace reader708 {
/**
 * @brief Reader class for ARINC 708 formatted radar data\n
 * Used for opening a 708 data file and reading in radar information
 * for each record.
 */
class read708 {
public:
/**
 * @brief read708 class constructor
 * @param _fileName: the full name of the data file (including path and extension)
 * @Throws Standard runtime error exception
 * 
 * Creates and instance of the read708 class, opens the data file for reading,
 * and allocates memory for data buffers. Throws a standard runtime error
 * exception if the file cannot be opened or memory allocation fails.
 */
    read708(string _fileName) throw (exception);
/**
 * @brief read708 class destructor
 * 
 * Closes the data file, frees data buffer memory, and deletes the class instance.
 */
    virtual ~read708();
/**
 * @brief Read a frame/record of data
 * @return Nothing
 * @Throws custom endOfFileException at end of file
 * 
 * This reads a frame of data and updates the CRframe class with the newest
 * values. It throws a custom endOfFileException when the end of the file is
 * reached.
 */
    void readFrame(void) throw (exception);
/**
 * @brief Get azimuth
 * @return azimuth angle in degrees
 * 
 * This gets the azimuth angle (scan angle) in degrees.
 */
    float getAzimuth(void);
/**
 * @brief Get elevation
 * @return elevation angle in degrees
 * 
 * This gets the elevation angle (tilt) in degrees.
 */
    float getElevation(void);
/**
 * @brief Get gain
 * @return gain in dB
 * 
 * This gets the gain setting in dB.
 */
    float getGain(void);
/**
 * @brief Get range
 * @return range in nautical miles
 * 
 * This gets the range setting in nautical miles.
 */
    float getRange(void);
/**
 * @brief Get display value
 * @param _bin: the range bin (0-511)
 * @return display value for the supplied range bin
 * 
 * This gets the display value for the supplied range bin.
 * 0 = black
 * 1 = green
 * 2 = yellow
 * 3 = red
 * 4 = magenta
 */
    unsigned int getDisplayValue(unsigned int _bin);
protected:
    FILE *in_flnme;
    unsigned int nDataWords; // number of 2 byte words in data block
    unsigned short *buffer;
    unsigned short headerData[HEADERWORDS];
    unsigned short *blockData;
    unsigned int displayValue[NRANGBINS];
    float azimuth, elevation, range, gain;
    // overloaded constructor for children classes to use to override format settings
    read708(string _fileName, unsigned int _nDataWords) throw (exception);
    void initialize(string _fileName) throw (exception);
    virtual void resetValues(void);
    void decodeHeader(void);
    virtual void determineGain(int _gainWord);
    virtual void decodeFrame(void);
};

/**
 * @brief Custom exception for end of a file
 */
class endOfFileException : public runtime_error {
public:
    endOfFileException() : runtime_error("end of file") {};
};
}
#endif	/* READ708_H */

